%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%                                %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%      DECLIPPING MAIN FILE      %%%%%%%%%%%%%%%%%%%%%%%     
%%%%%%%%%%%%%%%%%%%%                                %%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% using toolbox LTFAT

close all;
%clear variables;

%% input file settings   
% load audio-file
fprintf('Loading audio\n')
sound = 'acoustic_guitar.wav'; % 'acoustic_guitar.wav'
                               % 'double_bass.wav'
                               % 'piano_voice.wav'
                               % 'sample.wav'
                               % 'speech.wav'
[data, fs] = audioread(strcat('Sounds\', sound)); 

% normalization
maxAbsVal = max(abs(data));
if maxAbsVal < 0.95
    fprintf('Normalizing audio...')
    data = data/maxAbsVal;
    fprintf('(The original maximum was %f)\n', maxAbsVal)
end

% signal length
param.Ls = length(data);


%% settings
fprintf('Setting up the frame parameters\n')
param.theta = 0.1;    % set (symetric) clipping threshold 

% window parameters
param.w = 1024;       % window length
param.a = param.w/4;  % window shift
param.wtype = 'hann'; % options available on: http://ltfat.github.io/doc/sigproc/firwin.html

% DFT parameters
param.F = frame('dft');
param.F.redundancy = 2;  %non-native, our own additional parameter
param.F.frana = @(insig)dft([insig; zeros(length(insig)*(param.F.redundancy-1),1)]);
param.F.frsyn = @(insig)postpad(idft(insig),length(insig)/param.F.redundancy);

% paramsolver parameters
paramsolver.s = 1;   % increment of k
paramsolver.r = 1;   % every r-th iteration increment k by s   
paramsolver.epsilon = 0.1;  % stopping criterion of termination function
paramsolver.maxit = ceil(floor(param.w*param.F.redundancy/2+1)*paramsolver.r/paramsolver.s); % maximum number of iterations

paramsolver.store_sdr = 0; 
paramsolver.store_obj = 0;

param.algorithm = 'sspade_new'; % algorithm to compute declipping, options: 'aspade', 'sspade', 'sspade_new'
fprintf('Setting up algorithm: %s\n', param.algorithm)


%% clipping 
fprintf('Generating clipped signal\n')
[data_clipped, param.masks] = hard_clip(data, -param.theta, param.theta); % clipping the original signal


%% Main algorithm
fprintf('Starting the %s algorithm\n', param.algorithm)

tic; 

[data_rec, sdr_iter, obj_iter] = spade_segmentation(data_clipped, param, paramsolver, data);

time = toc;


%% Time & SNR evaluation

% time
fprintf('Result obtained in %4.3f seconds.\n', time);
% SNR
sdr_clip = sdr_n(data, data_clipped);
sdr_rec = sdr_n(data, data_rec);
fprintf('SDR of the clipped signal is %4.3f dB.\n', sdr_clip);
fprintf('SDR of the reconstructed signal is %4.3f dB.\n', sdr_rec);
fprintf('SDR improvement is %4.3f dB.\n', sdr_rec-sdr_clip);

